unit MsgBox;

{$F+,O+,X+}
interface

uses Objects,Dialogs;

const

{ Message box classes }

  mfWarning      = $0000;       { Display a Warning box }
  mfError        = $0001;       { Dispaly a Error box }
  mfInformation  = $0002;       { Display an Information Box }
  mfConfirmation = $0003;       { Display a Confirmation Box }
  efCritical     = $0004;
  efAccess       = $0005;
  efFatal        = $0006;
  efUser         = $0007;

{ Message box button flags }

  mfYesButton    = $0100;       { Put a Yes button into the dialog }
  mfNoButton     = $0200;       { Put a No button into the dialog }
  mfOKButton     = $0400;       { Put an OK button into the dialog }
  mfCancelButton = $0800;       { Put a Cancel button into the dialog }

  mfYesNoCancel  = mfYesButton + mfNoButton + mfCancelButton;
                                { Standard Yes, No, Cancel dialog }
  mfOKCancel     = mfOKButton + mfCancelButton;
                                { Standard OK, Cancel dialog }

  BoxContext:word=0;

function MessageBox(Msg: String; Params: Pointer; AOptions: Word): Word;

function MessageBoxRect(var R: TRect; Msg: String; Params: Pointer;
  AOptions: Word): Word;

function InputBox(Title: String; ALabel: String; var S: String;
  Limit: Byte): Word;

function InputBoxRect(var Bounds: TRect; Title: String; ALabel: String;
  var S: String;  Limit: Byte): Word;

function InfoBox(Msg: String): PDialog;

implementation
uses Drivers, Views, App;

function MessageBox(Msg: String; Params: Pointer;
                    AOptions: Word): Word;
var R: TRect;
begin
  R.Assign(0, 0, 40, 9);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  MessageBox := MessageBoxRect(R, Msg, Params, AOptions);
end;

function InfoBox(Msg: String): PDialog;
var Dialog: PDialog; Control: PView; R: TRect;
begin
  R.Assign(0, 0, 40, 6);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  Dialog := New(PDialog, Init(R, ''));
  with Dialog^ do begin
    Flags:=Flags and not wfClose;
    R.Assign(3, 2, Size.X - 2, Size.Y - 3);
    Control := New(PStaticText, Init(R, Msg));
    Insert(Control);
  end;
  InfoBox:=Dialog;
  Application^.Insert(Dialog);
end;

Procedure FlushBuffer;
Begin
 Inline($1E/$50/$B8/>$40/$8E/$D8/$FA/$A1/>$1A/$A3/>$1C/$FB/$58/$1F)
End;

function MessageBoxRect(var R: TRect; Msg: String; Params: Pointer;
                        AOptions: Word): Word;
const
  ButtonName: array[0..3] of string[6] =
    ('~Y~es', '~N~o', 'O~K~', '室');
  Commands: array[0..3] of word =
    (cmYes, cmNo, cmOK, cmCancel);
  Titles: array[0..6] of string[18] =
    ('।०','訡','ଠ','Confirm',
     '᪠ 訡', '訡 㯠', '⠫쭠 訡');
var
  I, X, ButtonCount: Integer;
  Dialog: PDialog; Control: PView; T: TRect;
  ButtonList: array[0..4] of PView; S: String; B:boolean;
begin
  Dialog := New(PDialog,
    Init(R, Titles[AOptions and $7]));
  with Dialog^ do begin
    R.Assign(3, 2, Size.X - 2, Size.Y - 3);
    FormatStr(S, Msg, Params^);
    Control := New(PStaticText, Init(R, S));
    Insert(Control);
    X := -2;
    ButtonCount := 0;
    for I := 0 to 3 do
      if AOptions and ($0100 shl I) <> 0 then begin
        R.Assign(0, 0, 10, 2);
        Control := New(PButton, Init(R, ButtonName[I], Commands[i],
          bfNormal));
        Inc(X, Control^.Size.X + 2);
        ButtonList[ButtonCount] := Control;
        Inc(ButtonCount);
      end;
    X := (Size.X - X) shr 1;
    for I := 0 to ButtonCount - 1 do begin
      Control := ButtonList[I];
      Insert(Control);
      Control^.MoveTo(X, Size.Y - 3);
      Inc(X, Control^.Size.X + 2);
    end;
    HelpCtx:=BoxContext;
    SelectNext(False);
  end;
  FlushBuffer;
  MessageBoxRect := DeskTop^.ExecView(Dialog);
  Dispose(Dialog, Done);
  BoxContext:=hcNoContext;
end;

function InputBox(Title: String; ALabel: String; var S: String;
                  Limit: Byte): Word;
var R: TRect;
begin
  R.Assign(0, 0, 60, 8);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  InputBox := InputBoxRect(R, Title, ALabel, S, Limit);
end;

function InputBoxRect(var Bounds: TRect; Title: String; ALabel: String;
                      var S: String;  Limit: Byte): Word;
var  Dialog: PDialog; Control: PView; R: TRect; C: Word; B:boolean;
begin
  Dialog := New(PDialog, Init(Bounds, Title));
  with Dialog^ do begin
    R.Assign(4 + CStrLen(ALabel), 2, Size.X - 3, 3);
    Control := New(PInputLine, Init(R, Limit));
    Insert(Control);
    R.Assign(2, 2, 3 + CStrLen(ALabel), 3);
    Insert(New(PLabel, Init(R, ALabel, Control)));
    R.Assign(Size.X - 24, Size.Y - 4, Size.X - 14, Size.Y - 2);
    Insert(New(PButton, Init(R, 'OK', cmOk, bfDefault)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    Insert(New(PButton, Init(R, '室', cmCancel, bfNormal)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    HelpCtx:=BoxContext;
    SelectNext(False);
  end;
  Dialog^.SetData(S);
  FlushBuffer;
  C := DeskTop^.ExecView(Dialog);
  if C <> cmCancel then Dialog^.GetData(S);
  Dispose(Dialog, Done);
  BoxContext:=hcNoContext;
  InputBoxRect := C;
end;

end.
